# -*- coding: utf-8 -*-

# Project Library
from gisdo.index import BaseIndex
from gisdo.algorithm import (
    EnrolledCountByFamily, EnrolledCountByHealth,
    EnrolledCountShortStay, EnrolledCountFullDayStay,
    EnrolledCountByCorrect, EnrolledCountByCare,
    EnrolledCountWantChangeDou, EnrolledCountByAge,
    EnrolledCountWantChangeDouWithHN, PredictableDecreaseContingent,
    EnrolledCountByDisabled)
from gisdo.constants import GOVERNMENT, DOU_TYPE_MAP, ALL_UNIT_TYPES
from gisdo.utils import AgeDeltas


class EnrolledIndex(BaseIndex):
    """
    Запросы группы "Количество детей зачисленных ... "
    """

    COUNT_BY_AGE = 1
    COUNT_DISABLED = 2
    COUNT_FAMILY = 3
    COUNT_CORRECT = 4
    COUNT_HEALTH = 5
    COUNT_SHORT_STAY = 6
    COUNT_FULL_DAY_STAY = 11
    COUNT_WANT_CHANGE_DOU = 7
    COUNT_WANT_CHANGE_DOU_WITH_HN = 8
    COUNT_BY_CARE = 9
    COUNT_DECREASE_CONTINGENT = 10

    ALGORITHM_MAP = {
        COUNT_BY_AGE: EnrolledCountByAge,
        COUNT_DISABLED: EnrolledCountByDisabled,
        COUNT_FAMILY: EnrolledCountByFamily,
        COUNT_CORRECT: EnrolledCountByCorrect,
        COUNT_HEALTH: EnrolledCountByHealth,
        COUNT_SHORT_STAY: EnrolledCountShortStay,
        COUNT_FULL_DAY_STAY: EnrolledCountFullDayStay,
        COUNT_WANT_CHANGE_DOU: EnrolledCountWantChangeDou,
        COUNT_WANT_CHANGE_DOU_WITH_HN: EnrolledCountWantChangeDouWithHN,
        COUNT_BY_CARE: EnrolledCountByCare,
        COUNT_DECREASE_CONTINGENT: PredictableDecreaseContingent
    }

    def get_count(
            self, dou_type=ALL_UNIT_TYPES,
            index_type=COUNT_BY_AGE, **kwargs):

        if dou_type != ALL_UNIT_TYPES:
            if ((not self.dou_type and dou_type != GOVERNMENT) or
                    (self.dou_type and
                     self.dou_type.code not in DOU_TYPE_MAP[dou_type])):
                return 0

        result_set = (
            EnrolledIndex.get_algorithm(
                index_type)(unit=self.unit, session=self.session)
            .get_result_set(**kwargs))

        return self.session.query(result_set)


def get_count(collection, age_filter):
    u"""Определение количества записей в коллекции подходящих под фильтр.

    Функция применяет фильтр age_filter к коллекции collection и возвращает
    количество отфильтрованных записей.
    """

    return len([
        1
        for _, _, _, date_of_birth in collection
        if age_filter(date_of_birth)
    ])


def get_age_filter(age, on_date):
    u"""Функция строит фильтр возраста."""

    down, up = AgeDeltas.get_category_deltas(age, on_date)

    filter_ = lambda rec: rec > down
    if up:
        if age == u'от 2 месяцев до 6 месяцев':
            filter_ = lambda rec: (down < rec)
        else:
            filter_ = lambda rec: (down < rec <= up)

    return filter_
